# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import same_doc_as

from ..models import ModelsBinaryRecordRequest
from ..models import ModelsGameBinaryRecordAdminResponse
from ..models import ModelsGameBinaryRecordCreate
from ..models import ModelsGameBinaryRecordMetadataRequest
from ..models import ModelsListGameBinaryRecordsAdminResponse
from ..models import ModelsResponseError
from ..models import ModelsUploadBinaryRecordRequest
from ..models import ModelsUploadBinaryRecordResponse

from ..operations.admin_game_binary_record import AdminDeleteGameBinaryRecordV1
from ..operations.admin_game_binary_record import AdminGetGameBinaryRecordV1
from ..operations.admin_game_binary_record import AdminListGameBinaryRecordsV1
from ..operations.admin_game_binary_record import AdminPostGameBinaryPresignedURLV1
from ..operations.admin_game_binary_record import AdminPostGameBinaryRecordV1
from ..operations.admin_game_binary_record import AdminPutGameBinaryRecordV1
from ..operations.admin_game_binary_record import AdminPutGameBinaryRecorMetadataV1
from ..models import ModelsGameBinaryRecordAdminResponseSetByEnum
from ..models import ModelsGameBinaryRecordCreateSetByEnum
from ..models import ModelsGameBinaryRecordMetadataRequestSetByEnum


@same_doc_as(AdminDeleteGameBinaryRecordV1)
def admin_delete_game_binary_record_v1(
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete game binary record (adminDeleteGameBinaryRecordV1)

    Delete a game binary record.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}

        method: DELETE

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Record deleted)

        400: Bad Request - ModelsResponseError (18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18322: record not found)

        500: Internal Server Error - ModelsResponseError (18320: unable to delete record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminDeleteGameBinaryRecordV1.create(
        key=key,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteGameBinaryRecordV1)
async def admin_delete_game_binary_record_v1_async(
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete game binary record (adminDeleteGameBinaryRecordV1)

    Delete a game binary record.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}

        method: DELETE

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Record deleted)

        400: Bad Request - ModelsResponseError (18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18322: record not found)

        500: Internal Server Error - ModelsResponseError (18320: unable to delete record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminDeleteGameBinaryRecordV1.create(
        key=key,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetGameBinaryRecordV1)
def admin_get_game_binary_record_v1(
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get game binary record (adminGetGameBinaryRecordV1)

    Get a game binary record by its key.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}

        method: GET

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsGameBinaryRecordAdminResponse (Record in namespace-level retrieved)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18303: record not found)

        500: Internal Server Error - ModelsResponseError (18301: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetGameBinaryRecordV1.create(
        key=key,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetGameBinaryRecordV1)
async def admin_get_game_binary_record_v1_async(
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get game binary record (adminGetGameBinaryRecordV1)

    Get a game binary record by its key.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}

        method: GET

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsGameBinaryRecordAdminResponse (Record in namespace-level retrieved)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18303: record not found)

        500: Internal Server Error - ModelsResponseError (18301: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetGameBinaryRecordV1.create(
        key=key,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminListGameBinaryRecordsV1)
def admin_list_game_binary_records_v1(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    query: Optional[str] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query game binary records (adminListGameBinaryRecordsV1)

    Retrieve list of binary records by namespace.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries

        method: GET

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        query: (query) OPTIONAL str in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelsListGameBinaryRecordsAdminResponse (Retrieve list of records by namespace)

        400: Bad Request - ModelsResponseError (18304: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18301: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminListGameBinaryRecordsV1.create(
        limit=limit,
        offset=offset,
        query=query,
        tags=tags,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminListGameBinaryRecordsV1)
async def admin_list_game_binary_records_v1_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    query: Optional[str] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query game binary records (adminListGameBinaryRecordsV1)

    Retrieve list of binary records by namespace.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries

        method: GET

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        query: (query) OPTIONAL str in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelsListGameBinaryRecordsAdminResponse (Retrieve list of records by namespace)

        400: Bad Request - ModelsResponseError (18304: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18301: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminListGameBinaryRecordsV1.create(
        limit=limit,
        offset=offset,
        query=query,
        tags=tags,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPostGameBinaryPresignedURLV1)
def admin_post_game_binary_presigned_urlv1(
    body: ModelsUploadBinaryRecordRequest,
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request presigned URL for upload game binary records (adminPostGameBinaryPresignedURLV1)

    Request presigned URL to upload the binary record to s3.

    Other detail info:

    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}/presigned

        method: POST

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUploadBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Successful Operation)

        400: Bad Request - ModelsResponseError (18311: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18313: record not found)

        500: Internal Server Error - ModelsResponseError (18312: unable to get record | 18314: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPostGameBinaryPresignedURLV1.create(
        body=body,
        key=key,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPostGameBinaryPresignedURLV1)
async def admin_post_game_binary_presigned_urlv1_async(
    body: ModelsUploadBinaryRecordRequest,
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request presigned URL for upload game binary records (adminPostGameBinaryPresignedURLV1)

    Request presigned URL to upload the binary record to s3.

    Other detail info:

    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}/presigned

        method: POST

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUploadBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Successful Operation)

        400: Bad Request - ModelsResponseError (18311: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18313: record not found)

        500: Internal Server Error - ModelsResponseError (18312: unable to get record | 18314: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPostGameBinaryPresignedURLV1.create(
        body=body,
        key=key,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPostGameBinaryRecordV1)
def admin_post_game_binary_record_v1(
    body: ModelsGameBinaryRecordCreate,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create game binary record (adminPostGameBinaryRecordV1)

    Create a game binary record.

    Other detail info:

    `key` should follow these rules:
    1. support uppercase and lowercase letters, numbers, and separators **"-"**, **"_"**, **"."** are allowed
    2. begin and end with letters or numbers
    3. spaces are not allowed
    4. separators must not appears twice in a row


    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries

        method: POST

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsGameBinaryRecordCreate in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Record in namespace-level created)

        400: Bad Request - ModelsResponseError (18305: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18309: key already exists)

        500: Internal Server Error - ModelsResponseError (18301: unable to get record | 18307: unable to save record | 18310: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPostGameBinaryRecordV1.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPostGameBinaryRecordV1)
async def admin_post_game_binary_record_v1_async(
    body: ModelsGameBinaryRecordCreate,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create game binary record (adminPostGameBinaryRecordV1)

    Create a game binary record.

    Other detail info:

    `key` should follow these rules:
    1. support uppercase and lowercase letters, numbers, and separators **"-"**, **"_"**, **"."** are allowed
    2. begin and end with letters or numbers
    3. spaces are not allowed
    4. separators must not appears twice in a row


    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries

        method: POST

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsGameBinaryRecordCreate in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Record in namespace-level created)

        400: Bad Request - ModelsResponseError (18305: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18309: key already exists)

        500: Internal Server Error - ModelsResponseError (18301: unable to get record | 18307: unable to save record | 18310: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPostGameBinaryRecordV1.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPutGameBinaryRecordV1)
def admin_put_game_binary_record_v1(
    body: ModelsBinaryRecordRequest,
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update game binary record file (adminPutGameBinaryRecordV1)

    Update a game binary record file by its key

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}

        method: PUT

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsGameBinaryRecordAdminResponse (Record saved)

        400: Bad Request - ModelsResponseError (18316: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18317: record not found)

        500: Internal Server Error - ModelsResponseError (18318: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPutGameBinaryRecordV1.create(
        body=body,
        key=key,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPutGameBinaryRecordV1)
async def admin_put_game_binary_record_v1_async(
    body: ModelsBinaryRecordRequest,
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update game binary record file (adminPutGameBinaryRecordV1)

    Update a game binary record file by its key

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}

        method: PUT

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsGameBinaryRecordAdminResponse (Record saved)

        400: Bad Request - ModelsResponseError (18316: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18317: record not found)

        500: Internal Server Error - ModelsResponseError (18318: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPutGameBinaryRecordV1.create(
        body=body,
        key=key,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminPutGameBinaryRecorMetadataV1)
def admin_put_game_binary_recor_metadata_v1(
    body: ModelsGameBinaryRecordMetadataRequest,
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update game binary record metadata (adminPutGameBinaryRecorMetadataV1)

    Update a game binary record metadata by its key

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}/metadata

        method: PUT

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsGameBinaryRecordMetadataRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsGameBinaryRecordAdminResponse (Record saved)

        400: Bad Request - ModelsResponseError (18316: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18317: record not found)

        500: Internal Server Error - ModelsResponseError (18318: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPutGameBinaryRecorMetadataV1.create(
        body=body,
        key=key,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminPutGameBinaryRecorMetadataV1)
async def admin_put_game_binary_recor_metadata_v1_async(
    body: ModelsGameBinaryRecordMetadataRequest,
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update game binary record metadata (adminPutGameBinaryRecorMetadataV1)

    Update a game binary record metadata by its key

    Properties:
        url: /cloudsave/v1/admin/namespaces/{namespace}/binaries/{key}/metadata

        method: PUT

        tags: ["AdminGameBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsGameBinaryRecordMetadataRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsGameBinaryRecordAdminResponse (Record saved)

        400: Bad Request - ModelsResponseError (18316: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18317: record not found)

        500: Internal Server Error - ModelsResponseError (18318: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminPutGameBinaryRecorMetadataV1.create(
        body=body,
        key=key,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation-init.j2

"""Auto-generated package that contains models used by the AccelByte Gaming Services Cloudsave Service."""

__version__ = "3.20.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .put_game_record_concurr_33e39a import PutGameRecordConcurrentHandlerV1
from .put_player_public_recor_3f1a7f import PutPlayerPublicRecordConcurrentHandlerV1
from .put_player_record_concu_385e05 import PutPlayerRecordConcurrentHandlerV1



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import same_doc_as

from ..models import ModelsBinaryRecordRequest
from ..models import ModelsBulkGetPlayerBinaryRecordResponse
from ..models import ModelsBulkGetPlayerRecordsRequest
from ..models import ModelsBulkUserIDsRequest
from ..models import ModelsListPlayerBinaryRecordsResponse
from ..models import ModelsPlayerBinaryRecordMetadataPublicRequest
from ..models import ModelsPlayerBinaryRecordResponse
from ..models import ModelsPublicPlayerBinaryRecordCreate
from ..models import ModelsResponseError
from ..models import ModelsUploadBinaryRecordRequest
from ..models import ModelsUploadBinaryRecordResponse

from ..operations.public_player_binary_record import BulkGetMyBinaryRecordV1
from ..operations.public_player_binary_record import (
    BulkGetOtherPlayerPublicBinaryRecordsV1,
)
from ..operations.public_player_binary_record import BulkGetPlayerPublicBinaryRecordsV1
from ..operations.public_player_binary_record import DeletePlayerBinaryRecordV1
from ..operations.public_player_binary_record import GetPlayerBinaryRecordV1
from ..operations.public_player_binary_record import GetPlayerPublicBinaryRecordsV1
from ..operations.public_player_binary_record import ListMyBinaryRecordsV1
from ..operations.public_player_binary_record import (
    ListOtherPlayerPublicBinaryRecordsV1,
)
from ..operations.public_player_binary_record import PostPlayerBinaryPresignedURLV1
from ..operations.public_player_binary_record import PostPlayerBinaryRecordV1
from ..operations.public_player_binary_record import PutPlayerBinaryRecordV1
from ..operations.public_player_binary_record import PutPlayerBinaryRecorMetadataV1
from ..models import ModelsPlayerBinaryRecordResponseSetByEnum


@same_doc_as(BulkGetMyBinaryRecordV1)
def bulk_get_my_binary_record_v1(
    body: ModelsBulkGetPlayerRecordsRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get player records bulk (bulkGetMyBinaryRecordV1)

    Retrieve player record key and payload in bulk under given namespace.
    Maximum bulk key limit per request 20

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/me/binaries/bulk

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBulkGetPlayerRecordsRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsBulkGetPlayerBinaryRecordResponse (Successful operation)

        400: Bad Request - ModelsResponseError (18125: invalid request body | 18126: request record keys list exceed max size [%d])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18343: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkGetMyBinaryRecordV1.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkGetMyBinaryRecordV1)
async def bulk_get_my_binary_record_v1_async(
    body: ModelsBulkGetPlayerRecordsRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get player records bulk (bulkGetMyBinaryRecordV1)

    Retrieve player record key and payload in bulk under given namespace.
    Maximum bulk key limit per request 20

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/me/binaries/bulk

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBulkGetPlayerRecordsRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsBulkGetPlayerBinaryRecordResponse (Successful operation)

        400: Bad Request - ModelsResponseError (18125: invalid request body | 18126: request record keys list exceed max size [%d])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18343: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkGetMyBinaryRecordV1.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(BulkGetOtherPlayerPublicBinaryRecordsV1)
def bulk_get_other_player_public_binary_records_v1(
    body: ModelsBulkGetPlayerRecordsRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk get other player public binary record (bulkGetOtherPlayerPublicBinaryRecordsV1)

    Retrieve other player public binary record in bulk under given namespace.
    Maximum bulk key limit per request 20

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/public/bulk

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBulkGetPlayerRecordsRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsBulkGetPlayerBinaryRecordResponse (Successful operation)

        400: Bad Request - ModelsResponseError (18342: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18343: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkGetOtherPlayerPublicBinaryRecordsV1.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkGetOtherPlayerPublicBinaryRecordsV1)
async def bulk_get_other_player_public_binary_records_v1_async(
    body: ModelsBulkGetPlayerRecordsRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk get other player public binary record (bulkGetOtherPlayerPublicBinaryRecordsV1)

    Retrieve other player public binary record in bulk under given namespace.
    Maximum bulk key limit per request 20

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/public/bulk

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBulkGetPlayerRecordsRequest in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsBulkGetPlayerBinaryRecordResponse (Successful operation)

        400: Bad Request - ModelsResponseError (18342: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18343: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkGetOtherPlayerPublicBinaryRecordsV1.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(BulkGetPlayerPublicBinaryRecordsV1)
def bulk_get_player_public_binary_records_v1(
    body: ModelsBulkUserIDsRequest,
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk get player public records (bulkGetPlayerPublicBinaryRecordsV1)

    Bulk get other player's public binary record by userIds, max allowed 20 at a time. Only record with `isPublic=true` can be
    retrieved using this endpoint.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/bulk/binaries/{key}/public

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBulkUserIDsRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsBulkGetPlayerBinaryRecordResponse (Record retrieved)

        400: Bad Request - ModelsResponseError (18342: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18343: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkGetPlayerPublicBinaryRecordsV1.create(
        body=body,
        key=key,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkGetPlayerPublicBinaryRecordsV1)
async def bulk_get_player_public_binary_records_v1_async(
    body: ModelsBulkUserIDsRequest,
    key: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk get player public records (bulkGetPlayerPublicBinaryRecordsV1)

    Bulk get other player's public binary record by userIds, max allowed 20 at a time. Only record with `isPublic=true` can be
    retrieved using this endpoint.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/bulk/binaries/{key}/public

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBulkUserIDsRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsBulkGetPlayerBinaryRecordResponse (Record retrieved)

        400: Bad Request - ModelsResponseError (18342: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18343: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkGetPlayerPublicBinaryRecordsV1.create(
        body=body,
        key=key,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeletePlayerBinaryRecordV1)
def delete_player_binary_record_v1(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete player binary record (deletePlayerBinaryRecordV1)

    Delete a player binary record. Only player who own the record can delete it

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: DELETE

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (Record deleted)

        400: Bad Request - ModelsResponseError (18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18338: record not found)

        500: Internal Server Error - ModelsResponseError (18336: unable to delete record | 18338: record not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeletePlayerBinaryRecordV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeletePlayerBinaryRecordV1)
async def delete_player_binary_record_v1_async(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete player binary record (deletePlayerBinaryRecordV1)

    Delete a player binary record. Only player who own the record can delete it

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: DELETE

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        204: No Content - (Record deleted)

        400: Bad Request - ModelsResponseError (18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18338: record not found)

        500: Internal Server Error - ModelsResponseError (18336: unable to delete record | 18338: record not found)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeletePlayerBinaryRecordV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPlayerBinaryRecordV1)
def get_player_binary_record_v1(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get player binary record (getPlayerBinaryRecordV1)

    Get a player binary record by its key.
    **Private Record**: Only user who own the record could retrieve it.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: GET

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record in user-level retrieved)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18325: record not found)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPlayerBinaryRecordV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPlayerBinaryRecordV1)
async def get_player_binary_record_v1_async(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get player binary record (getPlayerBinaryRecordV1)

    Get a player binary record by its key.
    **Private Record**: Only user who own the record could retrieve it.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: GET

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record in user-level retrieved)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18325: record not found)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPlayerBinaryRecordV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPlayerPublicBinaryRecordsV1)
def get_player_public_binary_records_v1(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get player public binary record (getPlayerPublicBinaryRecordsV1)

    Get other player's public binary record. Only record with `isPublic=true` can be
    retrieved using this endpoint.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}/public

        method: GET

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record retrieved)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18340: record not found)

        500: Internal Server Error - ModelsResponseError (18339: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPlayerPublicBinaryRecordsV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPlayerPublicBinaryRecordsV1)
async def get_player_public_binary_records_v1_async(
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get player public binary record (getPlayerPublicBinaryRecordsV1)

    Get other player's public binary record. Only record with `isPublic=true` can be
    retrieved using this endpoint.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}/public

        method: GET

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record retrieved)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18340: record not found)

        500: Internal Server Error - ModelsResponseError (18339: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPlayerPublicBinaryRecordsV1.create(
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListMyBinaryRecordsV1)
def list_my_binary_records_v1(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    query: Optional[str] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query my binary records (listMyBinaryRecordsV1)

    Retrieve list of my binary records by namespace.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/me/binaries

        method: GET

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        query: (query) OPTIONAL str in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelsListPlayerBinaryRecordsResponse (Retrieve list of player records by namespace)

        400: Bad Request - ModelsResponseError (18125: invalid request body | 18126: request record keys list exceed max size [%d])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18343: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListMyBinaryRecordsV1.create(
        limit=limit,
        offset=offset,
        query=query,
        tags=tags,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListMyBinaryRecordsV1)
async def list_my_binary_records_v1_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    query: Optional[str] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query my binary records (listMyBinaryRecordsV1)

    Retrieve list of my binary records by namespace.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/me/binaries

        method: GET

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        query: (query) OPTIONAL str in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelsListPlayerBinaryRecordsResponse (Retrieve list of player records by namespace)

        400: Bad Request - ModelsResponseError (18125: invalid request body | 18126: request record keys list exceed max size [%d])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18343: unable to get record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListMyBinaryRecordsV1.create(
        limit=limit,
        offset=offset,
        query=query,
        tags=tags,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListOtherPlayerPublicBinaryRecordsV1)
def list_other_player_public_binary_records_v1(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query other player public binary record (listOtherPlayerPublicBinaryRecordsV1)

    Retrieve list of other player public binary records under given namespace.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/public

        method: GET

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelsListPlayerBinaryRecordsResponse (Successful operation)

        400: Bad Request - ModelsResponseError (18347: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18345: unable to retrieve list of key records)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListOtherPlayerPublicBinaryRecordsV1.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        tags=tags,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListOtherPlayerPublicBinaryRecordsV1)
async def list_other_player_public_binary_records_v1_async(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query other player public binary record (listOtherPlayerPublicBinaryRecordsV1)

    Retrieve list of other player public binary records under given namespace.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/public

        method: GET

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelsListPlayerBinaryRecordsResponse (Successful operation)

        400: Bad Request - ModelsResponseError (18347: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        500: Internal Server Error - ModelsResponseError (18345: unable to retrieve list of key records)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListOtherPlayerPublicBinaryRecordsV1.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        tags=tags,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PostPlayerBinaryPresignedURLV1)
def post_player_binary_presigned_urlv1(
    body: ModelsUploadBinaryRecordRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request presigned URL for upload player binary records (postPlayerBinaryPresignedURLV1)

    Request presigned URL to upload the binary record to s3.

    Other detail info:

    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}/presigned

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUploadBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Successful Operation)

        400: Bad Request - ModelsResponseError (18311: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18313: record not found)

        500: Internal Server Error - ModelsResponseError (18312: unable to get record | 18314: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PostPlayerBinaryPresignedURLV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PostPlayerBinaryPresignedURLV1)
async def post_player_binary_presigned_urlv1_async(
    body: ModelsUploadBinaryRecordRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Request presigned URL for upload player binary records (postPlayerBinaryPresignedURLV1)

    Request presigned URL to upload the binary record to s3.

    Other detail info:

    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}/presigned

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsUploadBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Successful Operation)

        400: Bad Request - ModelsResponseError (18311: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18313: record not found)

        500: Internal Server Error - ModelsResponseError (18312: unable to get record | 18314: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PostPlayerBinaryPresignedURLV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PostPlayerBinaryRecordV1)
def post_player_binary_record_v1(
    body: ModelsPublicPlayerBinaryRecordCreate,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create player binary record (postPlayerBinaryRecordV1)

    Create a player binary record.

    Other detail info:

    `key` should follow these rules:
    1. support uppercase and lowercase letters, numbers, and separators **"-"**, **"_"**, **"."** are allowed
    2. begin and end with letters or numbers
    3. spaces are not allowed
    4. separators must not appears twice in a row


    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPublicPlayerBinaryRecordCreate in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Record in user-level created)

        400: Bad Request - ModelsResponseError (18327: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18330: key already exists)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record | 18328: unable to save record | 18331: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PostPlayerBinaryRecordV1.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PostPlayerBinaryRecordV1)
async def post_player_binary_record_v1_async(
    body: ModelsPublicPlayerBinaryRecordCreate,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create player binary record (postPlayerBinaryRecordV1)

    Create a player binary record.

    Other detail info:

    `key` should follow these rules:
    1. support uppercase and lowercase letters, numbers, and separators **"-"**, **"_"**, **"."** are allowed
    2. begin and end with letters or numbers
    3. spaces are not allowed
    4. separators must not appears twice in a row


    Supported file types: jpeg, jpg, png, bmp, gif, mp3, webp, and bin.

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries

        method: POST

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPublicPlayerBinaryRecordCreate in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - ModelsUploadBinaryRecordResponse (Record in user-level created)

        400: Bad Request - ModelsResponseError (18327: invalid request body)

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        409: Conflict - ModelsResponseError (18330: key already exists)

        500: Internal Server Error - ModelsResponseError (18323: unable to get record | 18328: unable to save record | 18331: unable to get presigned URL)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PostPlayerBinaryRecordV1.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PutPlayerBinaryRecordV1)
def put_player_binary_record_v1(
    body: ModelsBinaryRecordRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update player binary record file (putPlayerBinaryRecordV1)

    Update a player binary record file by its key

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: PUT

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record saved)

        400: Bad Request - ModelsResponseError (18332: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18333: record not found)

        500: Internal Server Error - ModelsResponseError (18334: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PutPlayerBinaryRecordV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PutPlayerBinaryRecordV1)
async def put_player_binary_record_v1_async(
    body: ModelsBinaryRecordRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update player binary record file (putPlayerBinaryRecordV1)

    Update a player binary record file by its key

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}

        method: PUT

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsBinaryRecordRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record saved)

        400: Bad Request - ModelsResponseError (18332: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18333: record not found)

        500: Internal Server Error - ModelsResponseError (18334: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PutPlayerBinaryRecordV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PutPlayerBinaryRecorMetadataV1)
def put_player_binary_recor_metadata_v1(
    body: ModelsPlayerBinaryRecordMetadataPublicRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update player binary record metadata (putPlayerBinaryRecorMetadataV1)

    Update a player binary record metadata by its key

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}/metadata

        method: PUT

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPlayerBinaryRecordMetadataPublicRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record saved)

        400: Bad Request - ModelsResponseError (18332: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18333: record not found)

        500: Internal Server Error - ModelsResponseError (18334: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PutPlayerBinaryRecorMetadataV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PutPlayerBinaryRecorMetadataV1)
async def put_player_binary_recor_metadata_v1_async(
    body: ModelsPlayerBinaryRecordMetadataPublicRequest,
    key: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update player binary record metadata (putPlayerBinaryRecorMetadataV1)

    Update a player binary record metadata by its key

    Properties:
        url: /cloudsave/v1/namespaces/{namespace}/users/{userId}/binaries/{key}/metadata

        method: PUT

        tags: ["PublicPlayerBinaryRecord"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelsPlayerBinaryRecordMetadataPublicRequest in body

        key: (key) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - ModelsPlayerBinaryRecordResponse (Record saved)

        400: Bad Request - ModelsResponseError (18332: invalid request body | 18201: invalid record operator, expect [%s] but actual [%s])

        401: Unauthorized - ModelsResponseError (20001: unauthorized access)

        403: Forbidden - ModelsResponseError (20013: insufficient permission)

        404: Not Found - ModelsResponseError (18333: record not found)

        500: Internal Server Error - ModelsResponseError (18334: unable to update record)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PutPlayerBinaryRecorMetadataV1.create(
        body=body,
        key=key,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation-init.j2

"""Auto-generated package that contains models used by the AccelByte Gaming Services Cloudsave Service."""

__version__ = "3.20.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_bulk_get_player_r_82c596 import AdminBulkGetPlayerRecordsByUserIDsHandlerV1
from .admin_bulk_put_player_r_27d5a1 import AdminBulkPutPlayerRecordsByKeyHandlerV1
from .admin_delete_player_pub_f11fd7 import AdminDeletePlayerPublicRecordHandlerV1
from .admin_delete_player_rec_168241 import AdminDeletePlayerRecordHandlerV1
from .admin_get_player_public_f015b6 import AdminGetPlayerPublicRecordHandlerV1
from .admin_get_player_record_b37faa import AdminGetPlayerRecordHandlerV1
from .admin_get_player_record_f4a5eb import AdminGetPlayerRecordsHandlerV1
from .admin_get_player_record_d622c9 import AdminGetPlayerRecordSizeHandlerV1
from .admin_post_player_publi_98e39e import AdminPostPlayerPublicRecordHandlerV1
from .admin_post_player_recor_6a96c8 import AdminPostPlayerRecordHandlerV1
from .admin_put_player_public_819faf import AdminPutPlayerPublicRecordHandlerV1
from .admin_put_player_record_a8195d import AdminPutPlayerRecordHandlerV1
from .admin_put_player_record_253e92 import AdminPutPlayerRecordsHandlerV1
from .admin_retrieve_player_records import AdminRetrievePlayerRecords
from .bulk_get_player_record__4627ec import BulkGetPlayerRecordSizeHandlerV1
from .list_player_record_handler_v1 import ListPlayerRecordHandlerV1



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Cloudsave Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class ModelsPublicGameBinaryRecordCreate(Model):
    """Models public game binary record create (models.PublicGameBinaryRecordCreate)

    Properties:
        file_type: (file_type) REQUIRED str

        key: (key) REQUIRED str
    """

    # region fields

    file_type: str  # REQUIRED
    key: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_file_type(self, value: str) -> ModelsPublicGameBinaryRecordCreate:
        self.file_type = value
        return self

    def with_key(self, value: str) -> ModelsPublicGameBinaryRecordCreate:
        self.key = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "file_type"):
            result["file_type"] = str(self.file_type)
        elif include_empty:
            result["file_type"] = ""
        if hasattr(self, "key"):
            result["key"] = str(self.key)
        elif include_empty:
            result["key"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls, file_type: str, key: str, **kwargs
    ) -> ModelsPublicGameBinaryRecordCreate:
        instance = cls()
        instance.file_type = file_type
        instance.key = key
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsPublicGameBinaryRecordCreate:
        instance = cls()
        if not dict_:
            return instance
        if "file_type" in dict_ and dict_["file_type"] is not None:
            instance.file_type = str(dict_["file_type"])
        elif include_empty:
            instance.file_type = ""
        if "key" in dict_ and dict_["key"] is not None:
            instance.key = str(dict_["key"])
        elif include_empty:
            instance.key = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsPublicGameBinaryRecordCreate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsPublicGameBinaryRecordCreate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsPublicGameBinaryRecordCreate,
        List[ModelsPublicGameBinaryRecordCreate],
        Dict[Any, ModelsPublicGameBinaryRecordCreate],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "file_type": "file_type",
            "key": "key",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "file_type": True,
            "key": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation-init.j2

"""Auto-generated package that contains models used by the AccelByte Gaming Services Cloudsave Service."""

__version__ = "3.20.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .bulk_get_player_public__28e43c import BulkGetPlayerPublicRecordHandlerV1
from .delete_player_record_ha_3addde import DeletePlayerRecordHandlerV1
from .get_other_player_public_17685a import GetOtherPlayerPublicRecordHandlerV1
from .get_other_player_public_4438d8 import GetOtherPlayerPublicRecordKeyHandlerV1
from .get_player_public_recor_5ea27d import GetPlayerPublicRecordHandlerV1
from .get_player_record_handler_v1 import GetPlayerRecordHandlerV1
from .get_player_records_bulk_b10df3 import GetPlayerRecordsBulkHandlerV1
from .post_player_public_reco_b8589e import PostPlayerPublicRecordHandlerV1
from .post_player_record_handler_v1 import PostPlayerRecordHandlerV1
from .public_delete_player_pu_0cc8b5 import PublicDeletePlayerPublicRecordHandlerV1
from .put_player_public_recor_7928cf import PutPlayerPublicRecordHandlerV1
from .put_player_record_handler_v1 import PutPlayerRecordHandlerV1
from .retrieve_player_records import RetrievePlayerRecords



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Cloudsave Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class ModelsBinaryInfoResponse(Model):
    """Models binary info response (models.BinaryInfoResponse)

    Properties:
        content_type: (content_type) REQUIRED str

        created_at: (created_at) REQUIRED str

        file_location: (file_location) REQUIRED str

        updated_at: (updated_at) REQUIRED str

        version: (version) REQUIRED int

        url: (url) OPTIONAL str
    """

    # region fields

    content_type: str  # REQUIRED
    created_at: str  # REQUIRED
    file_location: str  # REQUIRED
    updated_at: str  # REQUIRED
    version: int  # REQUIRED
    url: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_content_type(self, value: str) -> ModelsBinaryInfoResponse:
        self.content_type = value
        return self

    def with_created_at(self, value: str) -> ModelsBinaryInfoResponse:
        self.created_at = value
        return self

    def with_file_location(self, value: str) -> ModelsBinaryInfoResponse:
        self.file_location = value
        return self

    def with_updated_at(self, value: str) -> ModelsBinaryInfoResponse:
        self.updated_at = value
        return self

    def with_version(self, value: int) -> ModelsBinaryInfoResponse:
        self.version = value
        return self

    def with_url(self, value: str) -> ModelsBinaryInfoResponse:
        self.url = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "content_type"):
            result["content_type"] = str(self.content_type)
        elif include_empty:
            result["content_type"] = ""
        if hasattr(self, "created_at"):
            result["created_at"] = str(self.created_at)
        elif include_empty:
            result["created_at"] = ""
        if hasattr(self, "file_location"):
            result["file_location"] = str(self.file_location)
        elif include_empty:
            result["file_location"] = ""
        if hasattr(self, "updated_at"):
            result["updated_at"] = str(self.updated_at)
        elif include_empty:
            result["updated_at"] = ""
        if hasattr(self, "version"):
            result["version"] = int(self.version)
        elif include_empty:
            result["version"] = 0
        if hasattr(self, "url"):
            result["url"] = str(self.url)
        elif include_empty:
            result["url"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        content_type: str,
        created_at: str,
        file_location: str,
        updated_at: str,
        version: int,
        url: Optional[str] = None,
        **kwargs,
    ) -> ModelsBinaryInfoResponse:
        instance = cls()
        instance.content_type = content_type
        instance.created_at = created_at
        instance.file_location = file_location
        instance.updated_at = updated_at
        instance.version = version
        if url is not None:
            instance.url = url
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsBinaryInfoResponse:
        instance = cls()
        if not dict_:
            return instance
        if "content_type" in dict_ and dict_["content_type"] is not None:
            instance.content_type = str(dict_["content_type"])
        elif include_empty:
            instance.content_type = ""
        if "created_at" in dict_ and dict_["created_at"] is not None:
            instance.created_at = str(dict_["created_at"])
        elif include_empty:
            instance.created_at = ""
        if "file_location" in dict_ and dict_["file_location"] is not None:
            instance.file_location = str(dict_["file_location"])
        elif include_empty:
            instance.file_location = ""
        if "updated_at" in dict_ and dict_["updated_at"] is not None:
            instance.updated_at = str(dict_["updated_at"])
        elif include_empty:
            instance.updated_at = ""
        if "version" in dict_ and dict_["version"] is not None:
            instance.version = int(dict_["version"])
        elif include_empty:
            instance.version = 0
        if "url" in dict_ and dict_["url"] is not None:
            instance.url = str(dict_["url"])
        elif include_empty:
            instance.url = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsBinaryInfoResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsBinaryInfoResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsBinaryInfoResponse,
        List[ModelsBinaryInfoResponse],
        Dict[Any, ModelsBinaryInfoResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "content_type": "content_type",
            "created_at": "created_at",
            "file_location": "file_location",
            "updated_at": "updated_at",
            "version": "version",
            "url": "url",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "content_type": True,
            "created_at": True,
            "file_location": True,
            "updated_at": True,
            "version": True,
            "url": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Cloudsave Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class ModelsCustomFunction(Model):
    """Models custom function (models.CustomFunction)

    Properties:
        after_bulk_read_game_binary_record: (afterBulkReadGameBinaryRecord) REQUIRED bool

        after_bulk_read_game_record: (afterBulkReadGameRecord) REQUIRED bool

        after_bulk_read_player_binary_record: (afterBulkReadPlayerBinaryRecord) REQUIRED bool

        after_bulk_read_player_record: (afterBulkReadPlayerRecord) REQUIRED bool

        after_read_game_binary_record: (afterReadGameBinaryRecord) REQUIRED bool

        after_read_game_record: (afterReadGameRecord) REQUIRED bool

        after_read_player_binary_record: (afterReadPlayerBinaryRecord) REQUIRED bool

        after_read_player_record: (afterReadPlayerRecord) REQUIRED bool

        before_write_admin_game_record: (beforeWriteAdminGameRecord) REQUIRED bool

        before_write_admin_player_record: (beforeWriteAdminPlayerRecord) REQUIRED bool

        before_write_game_binary_record: (beforeWriteGameBinaryRecord) REQUIRED bool

        before_write_game_record: (beforeWriteGameRecord) REQUIRED bool

        before_write_player_binary_record: (beforeWritePlayerBinaryRecord) REQUIRED bool

        before_write_player_record: (beforeWritePlayerRecord) REQUIRED bool
    """

    # region fields

    after_bulk_read_game_binary_record: bool  # REQUIRED
    after_bulk_read_game_record: bool  # REQUIRED
    after_bulk_read_player_binary_record: bool  # REQUIRED
    after_bulk_read_player_record: bool  # REQUIRED
    after_read_game_binary_record: bool  # REQUIRED
    after_read_game_record: bool  # REQUIRED
    after_read_player_binary_record: bool  # REQUIRED
    after_read_player_record: bool  # REQUIRED
    before_write_admin_game_record: bool  # REQUIRED
    before_write_admin_player_record: bool  # REQUIRED
    before_write_game_binary_record: bool  # REQUIRED
    before_write_game_record: bool  # REQUIRED
    before_write_player_binary_record: bool  # REQUIRED
    before_write_player_record: bool  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_after_bulk_read_game_binary_record(
        self, value: bool
    ) -> ModelsCustomFunction:
        self.after_bulk_read_game_binary_record = value
        return self

    def with_after_bulk_read_game_record(self, value: bool) -> ModelsCustomFunction:
        self.after_bulk_read_game_record = value
        return self

    def with_after_bulk_read_player_binary_record(
        self, value: bool
    ) -> ModelsCustomFunction:
        self.after_bulk_read_player_binary_record = value
        return self

    def with_after_bulk_read_player_record(self, value: bool) -> ModelsCustomFunction:
        self.after_bulk_read_player_record = value
        return self

    def with_after_read_game_binary_record(self, value: bool) -> ModelsCustomFunction:
        self.after_read_game_binary_record = value
        return self

    def with_after_read_game_record(self, value: bool) -> ModelsCustomFunction:
        self.after_read_game_record = value
        return self

    def with_after_read_player_binary_record(self, value: bool) -> ModelsCustomFunction:
        self.after_read_player_binary_record = value
        return self

    def with_after_read_player_record(self, value: bool) -> ModelsCustomFunction:
        self.after_read_player_record = value
        return self

    def with_before_write_admin_game_record(self, value: bool) -> ModelsCustomFunction:
        self.before_write_admin_game_record = value
        return self

    def with_before_write_admin_player_record(
        self, value: bool
    ) -> ModelsCustomFunction:
        self.before_write_admin_player_record = value
        return self

    def with_before_write_game_binary_record(self, value: bool) -> ModelsCustomFunction:
        self.before_write_game_binary_record = value
        return self

    def with_before_write_game_record(self, value: bool) -> ModelsCustomFunction:
        self.before_write_game_record = value
        return self

    def with_before_write_player_binary_record(
        self, value: bool
    ) -> ModelsCustomFunction:
        self.before_write_player_binary_record = value
        return self

    def with_before_write_player_record(self, value: bool) -> ModelsCustomFunction:
        self.before_write_player_record = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "after_bulk_read_game_binary_record"):
            result["afterBulkReadGameBinaryRecord"] = bool(
                self.after_bulk_read_game_binary_record
            )
        elif include_empty:
            result["afterBulkReadGameBinaryRecord"] = False
        if hasattr(self, "after_bulk_read_game_record"):
            result["afterBulkReadGameRecord"] = bool(self.after_bulk_read_game_record)
        elif include_empty:
            result["afterBulkReadGameRecord"] = False
        if hasattr(self, "after_bulk_read_player_binary_record"):
            result["afterBulkReadPlayerBinaryRecord"] = bool(
                self.after_bulk_read_player_binary_record
            )
        elif include_empty:
            result["afterBulkReadPlayerBinaryRecord"] = False
        if hasattr(self, "after_bulk_read_player_record"):
            result["afterBulkReadPlayerRecord"] = bool(
                self.after_bulk_read_player_record
            )
        elif include_empty:
            result["afterBulkReadPlayerRecord"] = False
        if hasattr(self, "after_read_game_binary_record"):
            result["afterReadGameBinaryRecord"] = bool(
                self.after_read_game_binary_record
            )
        elif include_empty:
            result["afterReadGameBinaryRecord"] = False
        if hasattr(self, "after_read_game_record"):
            result["afterReadGameRecord"] = bool(self.after_read_game_record)
        elif include_empty:
            result["afterReadGameRecord"] = False
        if hasattr(self, "after_read_player_binary_record"):
            result["afterReadPlayerBinaryRecord"] = bool(
                self.after_read_player_binary_record
            )
        elif include_empty:
            result["afterReadPlayerBinaryRecord"] = False
        if hasattr(self, "after_read_player_record"):
            result["afterReadPlayerRecord"] = bool(self.after_read_player_record)
        elif include_empty:
            result["afterReadPlayerRecord"] = False
        if hasattr(self, "before_write_admin_game_record"):
            result["beforeWriteAdminGameRecord"] = bool(
                self.before_write_admin_game_record
            )
        elif include_empty:
            result["beforeWriteAdminGameRecord"] = False
        if hasattr(self, "before_write_admin_player_record"):
            result["beforeWriteAdminPlayerRecord"] = bool(
                self.before_write_admin_player_record
            )
        elif include_empty:
            result["beforeWriteAdminPlayerRecord"] = False
        if hasattr(self, "before_write_game_binary_record"):
            result["beforeWriteGameBinaryRecord"] = bool(
                self.before_write_game_binary_record
            )
        elif include_empty:
            result["beforeWriteGameBinaryRecord"] = False
        if hasattr(self, "before_write_game_record"):
            result["beforeWriteGameRecord"] = bool(self.before_write_game_record)
        elif include_empty:
            result["beforeWriteGameRecord"] = False
        if hasattr(self, "before_write_player_binary_record"):
            result["beforeWritePlayerBinaryRecord"] = bool(
                self.before_write_player_binary_record
            )
        elif include_empty:
            result["beforeWritePlayerBinaryRecord"] = False
        if hasattr(self, "before_write_player_record"):
            result["beforeWritePlayerRecord"] = bool(self.before_write_player_record)
        elif include_empty:
            result["beforeWritePlayerRecord"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        after_bulk_read_game_binary_record: bool,
        after_bulk_read_game_record: bool,
        after_bulk_read_player_binary_record: bool,
        after_bulk_read_player_record: bool,
        after_read_game_binary_record: bool,
        after_read_game_record: bool,
        after_read_player_binary_record: bool,
        after_read_player_record: bool,
        before_write_admin_game_record: bool,
        before_write_admin_player_record: bool,
        before_write_game_binary_record: bool,
        before_write_game_record: bool,
        before_write_player_binary_record: bool,
        before_write_player_record: bool,
        **kwargs,
    ) -> ModelsCustomFunction:
        instance = cls()
        instance.after_bulk_read_game_binary_record = after_bulk_read_game_binary_record
        instance.after_bulk_read_game_record = after_bulk_read_game_record
        instance.after_bulk_read_player_binary_record = (
            after_bulk_read_player_binary_record
        )
        instance.after_bulk_read_player_record = after_bulk_read_player_record
        instance.after_read_game_binary_record = after_read_game_binary_record
        instance.after_read_game_record = after_read_game_record
        instance.after_read_player_binary_record = after_read_player_binary_record
        instance.after_read_player_record = after_read_player_record
        instance.before_write_admin_game_record = before_write_admin_game_record
        instance.before_write_admin_player_record = before_write_admin_player_record
        instance.before_write_game_binary_record = before_write_game_binary_record
        instance.before_write_game_record = before_write_game_record
        instance.before_write_player_binary_record = before_write_player_binary_record
        instance.before_write_player_record = before_write_player_record
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsCustomFunction:
        instance = cls()
        if not dict_:
            return instance
        if (
            "afterBulkReadGameBinaryRecord" in dict_
            and dict_["afterBulkReadGameBinaryRecord"] is not None
        ):
            instance.after_bulk_read_game_binary_record = bool(
                dict_["afterBulkReadGameBinaryRecord"]
            )
        elif include_empty:
            instance.after_bulk_read_game_binary_record = False
        if (
            "afterBulkReadGameRecord" in dict_
            and dict_["afterBulkReadGameRecord"] is not None
        ):
            instance.after_bulk_read_game_record = bool(
                dict_["afterBulkReadGameRecord"]
            )
        elif include_empty:
            instance.after_bulk_read_game_record = False
        if (
            "afterBulkReadPlayerBinaryRecord" in dict_
            and dict_["afterBulkReadPlayerBinaryRecord"] is not None
        ):
            instance.after_bulk_read_player_binary_record = bool(
                dict_["afterBulkReadPlayerBinaryRecord"]
            )
        elif include_empty:
            instance.after_bulk_read_player_binary_record = False
        if (
            "afterBulkReadPlayerRecord" in dict_
            and dict_["afterBulkReadPlayerRecord"] is not None
        ):
            instance.after_bulk_read_player_record = bool(
                dict_["afterBulkReadPlayerRecord"]
            )
        elif include_empty:
            instance.after_bulk_read_player_record = False
        if (
            "afterReadGameBinaryRecord" in dict_
            and dict_["afterReadGameBinaryRecord"] is not None
        ):
            instance.after_read_game_binary_record = bool(
                dict_["afterReadGameBinaryRecord"]
            )
        elif include_empty:
            instance.after_read_game_binary_record = False
        if "afterReadGameRecord" in dict_ and dict_["afterReadGameRecord"] is not None:
            instance.after_read_game_record = bool(dict_["afterReadGameRecord"])
        elif include_empty:
            instance.after_read_game_record = False
        if (
            "afterReadPlayerBinaryRecord" in dict_
            and dict_["afterReadPlayerBinaryRecord"] is not None
        ):
            instance.after_read_player_binary_record = bool(
                dict_["afterReadPlayerBinaryRecord"]
            )
        elif include_empty:
            instance.after_read_player_binary_record = False
        if (
            "afterReadPlayerRecord" in dict_
            and dict_["afterReadPlayerRecord"] is not None
        ):
            instance.after_read_player_record = bool(dict_["afterReadPlayerRecord"])
        elif include_empty:
            instance.after_read_player_record = False
        if (
            "beforeWriteAdminGameRecord" in dict_
            and dict_["beforeWriteAdminGameRecord"] is not None
        ):
            instance.before_write_admin_game_record = bool(
                dict_["beforeWriteAdminGameRecord"]
            )
        elif include_empty:
            instance.before_write_admin_game_record = False
        if (
            "beforeWriteAdminPlayerRecord" in dict_
            and dict_["beforeWriteAdminPlayerRecord"] is not None
        ):
            instance.before_write_admin_player_record = bool(
                dict_["beforeWriteAdminPlayerRecord"]
            )
        elif include_empty:
            instance.before_write_admin_player_record = False
        if (
            "beforeWriteGameBinaryRecord" in dict_
            and dict_["beforeWriteGameBinaryRecord"] is not None
        ):
            instance.before_write_game_binary_record = bool(
                dict_["beforeWriteGameBinaryRecord"]
            )
        elif include_empty:
            instance.before_write_game_binary_record = False
        if (
            "beforeWriteGameRecord" in dict_
            and dict_["beforeWriteGameRecord"] is not None
        ):
            instance.before_write_game_record = bool(dict_["beforeWriteGameRecord"])
        elif include_empty:
            instance.before_write_game_record = False
        if (
            "beforeWritePlayerBinaryRecord" in dict_
            and dict_["beforeWritePlayerBinaryRecord"] is not None
        ):
            instance.before_write_player_binary_record = bool(
                dict_["beforeWritePlayerBinaryRecord"]
            )
        elif include_empty:
            instance.before_write_player_binary_record = False
        if (
            "beforeWritePlayerRecord" in dict_
            and dict_["beforeWritePlayerRecord"] is not None
        ):
            instance.before_write_player_record = bool(dict_["beforeWritePlayerRecord"])
        elif include_empty:
            instance.before_write_player_record = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsCustomFunction]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsCustomFunction]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsCustomFunction,
        List[ModelsCustomFunction],
        Dict[Any, ModelsCustomFunction],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "afterBulkReadGameBinaryRecord": "after_bulk_read_game_binary_record",
            "afterBulkReadGameRecord": "after_bulk_read_game_record",
            "afterBulkReadPlayerBinaryRecord": "after_bulk_read_player_binary_record",
            "afterBulkReadPlayerRecord": "after_bulk_read_player_record",
            "afterReadGameBinaryRecord": "after_read_game_binary_record",
            "afterReadGameRecord": "after_read_game_record",
            "afterReadPlayerBinaryRecord": "after_read_player_binary_record",
            "afterReadPlayerRecord": "after_read_player_record",
            "beforeWriteAdminGameRecord": "before_write_admin_game_record",
            "beforeWriteAdminPlayerRecord": "before_write_admin_player_record",
            "beforeWriteGameBinaryRecord": "before_write_game_binary_record",
            "beforeWriteGameRecord": "before_write_game_record",
            "beforeWritePlayerBinaryRecord": "before_write_player_binary_record",
            "beforeWritePlayerRecord": "before_write_player_record",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "afterBulkReadGameBinaryRecord": True,
            "afterBulkReadGameRecord": True,
            "afterBulkReadPlayerBinaryRecord": True,
            "afterBulkReadPlayerRecord": True,
            "afterReadGameBinaryRecord": True,
            "afterReadGameRecord": True,
            "afterReadPlayerBinaryRecord": True,
            "afterReadPlayerRecord": True,
            "beforeWriteAdminGameRecord": True,
            "beforeWriteAdminPlayerRecord": True,
            "beforeWriteGameBinaryRecord": True,
            "beforeWriteGameRecord": True,
            "beforeWritePlayerBinaryRecord": True,
            "beforeWritePlayerRecord": True,
        }

    # endregion static methods
